@@ -59,6 +59,7 @@ pub struct CapacityConstraintModule<T: LoadOps> {
     code: i32,
     state_keys: Vec<i32>,
     conditional: ConditionalJobModule,
+    activity: Arc<dyn ActivityCost + Send + Sync>,
     transport: Arc<dyn TransportCost + Send + Sync>,
     constraints: Vec<ConstraintVariant>,
     multi_trip: Arc<dyn MultiTrip<T> + Send + Sync>,
@@ -66,12 +67,17 @@ pub struct CapacityConstraintModule<T: LoadOps> {
 
 impl<T: LoadOps + 'static> CapacityConstraintModule<T> {
     /// Creates a new instance of `CapacityConstraintModule` without multi trip (reload) functionality
-    pub fn new(transport: Arc<dyn TransportCost + Send + Sync>, code: i32) -> Self {
-        Self::new_with_multi_trip(transport, code, Arc::new(NoMultiTrip { phantom: PhantomData }))
+    pub fn new(
+        activity: Arc<dyn ActivityCost + Send + Sync>,
+        transport: Arc<dyn TransportCost + Send + Sync>,
+        code: i32,
+    ) -> Self {
+        Self::new_with_multi_trip(activity, transport, code, Arc::new(NoMultiTrip { phantom: PhantomData }))
     }
 
     /// Creates a new instance of `CapacityConstraintModule` with multi trip (reload) functionality
     pub fn new_with_multi_trip(
+        activity: Arc<dyn ActivityCost + Send + Sync>,
         transport: Arc<dyn TransportCost + Send + Sync>,
         code: i32,
         multi_trip: Arc<dyn MultiTrip<T> + Send + Sync>,
@@ -91,6 +97,7 @@ impl<T: LoadOps + 'static> CapacityConstraintModule<T> {
                     move |_, _, job| multi_trip.is_reload_job(job)
                 },
             })),
+            activity,
             transport,
             constraints: vec![
                 ConstraintVariant::SoftRoute(Arc::new(CapacitySoftRouteConstraint { multi_trip: multi_trip.clone() })),
@@ -207,7 +214,7 @@ impl<T: LoadOps + 'static> CapacityConstraintModule<T> {
 
             if rc.is_stale() {
                 self.actualize_intervals(rc);
-                update_route_schedule(rc, self.transport.as_ref());
+                update_route_schedule(rc, self.activity.as_ref(), self.transport.as_ref());
             }
         });
         ctx.ignored.extend(extra_ignored.into_iter());
