@@ -1,7 +1,8 @@
 use crate::models::common::{Distance, Duration, Location, Schedule, TimeWindow};
 use crate::models::problem::{Actor, Job, Multi, Single};
 use crate::models::solution::Tour;
-use crate::utils::{compare_floats, compare_shared};
+use crate::utils::compare_shared;
+use rosomaxa::prelude::compare_floats;
 use std::cmp::Ordering;
 use std::sync::Arc;
 
