@@ -10,7 +10,8 @@ use rosomaxa::termination::*;
 use std::marker::PhantomData;
 
 /// A type alias for domain specific population.
-pub type TargetPopulation = Box<dyn HeuristicPopulation<Objective = ProblemObjective, Individual = InsertionContext>>;
+pub type TargetPopulation =
+    Box<dyn HeuristicPopulation<Objective = ProblemObjective, Individual = InsertionContext> + Send + Sync>;
 /// A type alias for domain specific heuristic.
 pub type TargetHeuristic =
     Box<dyn HyperHeuristic<Context = RefinementContext, Objective = ProblemObjective, Solution = InsertionContext>>;
@@ -29,7 +30,7 @@ pub type ElitismPopulation = Elitism<ProblemObjective, InsertionContext>;
 pub type RosomaxaPopulation = Rosomaxa<ProblemObjective, InsertionContext>;
 
 /// A type alias for domain specific termination type.
-pub type TargetTermination = dyn Termination<Context = RefinementContext, Objective = ProblemObjective> + Send + Sync;
+pub type DynTermination = dyn Termination<Context = RefinementContext, Objective = ProblemObjective> + Send + Sync;
 /// A type for composite termination.
 pub type TargetCompositeTermination = CompositeTermination<RefinementContext, ProblemObjective, InsertionContext>;
 /// A type for max time termination.
@@ -49,15 +50,22 @@ pub type ProblemConfigBuilder = EvolutionConfigBuilder<RefinementContext, Proble
 
 /// Creates config builder with default settings.
 pub fn create_default_config_builder(problem: Arc<Problem>, environment: Arc<Environment>) -> ProblemConfigBuilder {
-    let population = get_default_population::<RefinementContext, _, _>(problem.objective.clone(), environment.clone());
+    let selection_size = get_default_selection_size(environment.as_ref());
+    let population = get_default_population(problem.objective.clone(), environment.clone(), selection_size);
+    let telemetry_mode = get_default_telemetry_mode(environment.logger.clone());
 
     ProblemConfigBuilder::default()
         .with_heuristic(get_default_heuristic(problem.clone(), environment.clone()))
-        .with_context(RefinementContext::new(problem.clone(), population, environment.clone()))
+        .with_context(RefinementContext::new(problem.clone(), population, telemetry_mode, environment.clone()))
         .with_initial(4, 0.05, create_default_init_operators(problem, environment))
         .with_processing(create_default_processing())
 }
 
+/// Creates default telemetry mode.B
+pub fn get_default_telemetry_mode(logger: InfoLogger) -> TelemetryMode {
+    TelemetryMode::OnlyLogging { logger, log_best: 100, log_population: 1000, dump_population: false }
+}
+
 /// Gets default heuristic.
 pub fn get_default_heuristic(problem: Arc<Problem>, environment: Arc<Environment>) -> TargetHeuristic {
     Box::new(MultiSelective::<RefinementContext, ProblemObjective, InsertionContext>::new(
@@ -164,7 +172,7 @@ pub fn create_context_operator_probability(
                 return false;
             }
 
-            let phase_probability = phases.get(&refinement_ctx.population.selection_phase()).cloned().unwrap_or(0.);
+            let phase_probability = phases.get(&refinement_ctx.population().selection_phase()).cloned().unwrap_or(0.);
             random.is_hit(phase_probability)
         }),
         PhantomData::default(),
