@@ -1,18 +1,16 @@
 use super::*;
 use crate::construction::heuristics::*;
-use crate::utils::{parallel_into_collect, CollectGroupBy};
+use rosomaxa::utils::{parallel_into_collect, CollectGroupBy};
 
 /// Tries to improve job unassignment reason.
 #[derive(Default)]
 pub struct UnassignmentReason {}
 
-impl Processing for UnassignmentReason {
-    fn pre_process(&self, problem: Arc<Problem>, _environment: Arc<Environment>) -> Arc<Problem> {
-        problem
-    }
+impl HeuristicSolutionProcessing for UnassignmentReason {
+    type Solution = InsertionContext;
 
-    fn post_process(&self, insertion_ctx: InsertionContext) -> InsertionContext {
-        let mut insertion_ctx = insertion_ctx;
+    fn post_process(&self, solution: Self::Solution) -> Self::Solution {
+        let mut insertion_ctx = solution;
 
         let unassigned = insertion_ctx.solution.unassigned.drain().collect::<Vec<_>>();
         let leg_selector = VariableLegSelector::new(insertion_ctx.environment.random.clone());
