@@ -1,4 +1,5 @@
-use rand::prelude::*;
+use rand::prelude::StdRng;
+use rand::SeedableRng;
 use rosomaxa::prelude::Random;
 
 struct FakeDistribution<T> {
